/**
 ** imuCntrlTaskCodels.c
 **
 ** Codels used by the control task imuCntrlTask
 **
 ** Auteurs: Anthony Mallet & Thierry Peynot
 ** Date: 2002-2004
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#else
#include <portLib.h>
#endif
#include <stdio.h>
#include "server/imuHeader.h"

#include "imuInt.h"
#include "imuStruct.h"


/*------------------------------------------------------------------------
 * imuDumpImuDataOnFileCntrl  -  control codel of EXEC request DumpImuDataOnFile
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imu_BAD_PARAMS
 *
 * Return:    OK ou ERROR
 */

STATUS imuDumpImuDataOnFileCntrl (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{
  /** (IMU_DUMPPERIOD_MIN*Periode de la tache d'exec) devrait etre egal  la periode de lecture des donnees sur le port serie cf la requete Init et la tache imuReadTaskPerm */
  if(dumpOnFileParams->dumpPeriod < IMU_DUMPPERIOD_MIN)
    {
      printf("imuDumpImuDataOnFileCntrl: dumpPeriod must be >= %d", IMU_DUMPPERIOD_MIN);
      *bilan=S_imu_BAD_PARAMS;
      return ERROR;
    }

  return OK;
}



/*------------------------------------------------------------------------
 * imuSetVariancesCntrl  -  control codel of CONTROL request SetVariances
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imuCntrlTask_BAD_PARAMS
 *
 * Return:    OK ou ERROR
 */

STATUS imuSetVariancesCntrl (IMU_VARIANCES *imuVariances, int *bilan)
{
  /* ... add your code here ... */
  /* Y a-t-il quelque chose a controler ici ? */
  return OK;
}



/*------------------------------------------------------------------------
 * imuFilterImuDataCntrl  -  control codel of CONTROL request FilterImuData
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imu_BAD_PARAMS
 *
 * Return:    OK ou ERROR
 */

STATUS imuFilterImuDataCntrl (IMU_FILTER_PARAMS *imuDataFilterParams, int *bilan)
{

  /*-- Test des parametres ------------------*/
  if( (imuDataFilterParams->angularSpeedsFilterMode != IMU_FILTER_OFF)&&(imuDataFilterParams->angularSpeedsFilterMode != IMU_FILTER_ON) )
    {
      *bilan = S_imu_BAD_PARAMS ;
      printf("imuFilterAngularSpeedsCntrl: unexpected value for angularSpeedsFilterMode\n");
      return ERROR;
    }
  
  if( imuDataFilterParams->numSamplesUsedToFilter_angSpeeds == 0 )
    {
      *bilan = S_imu_BAD_PARAMS;
      printf("imuFilterAngularSpeedsCntrl: numSamplesUsedToFilter should be > 0 to be useful!\n");
      return ERROR;
    }

  /*-- Activation du filtre vitesses angulaires si demand ------------*/
  if(imuDataFilterParams->angularSpeedsFilterMode == IMU_FILTER_ON)
    {
      if( imuFilterInit(imuDataFilterParams->numSamplesUsedToFilter_angSpeeds,
			imuDataFilterParams->numSamplesUsedToFilter_angSpeeds/2.0) != OK)
	{
	  *bilan = S_imu_BAD_PARAMS;
	  return ERROR;
	}
    }

 
  /*-- Activation du filtre vitesses angulaires si demand ------------*/
  if(imuDataFilterParams->accelFilterMode == IMU_FILTER_ON)
    {
      if( imuFilterInit_accel(imuDataFilterParams->numSamplesUsedToFilter_accel,
			      imuDataFilterParams->numSamplesUsedToFilter_accel/2.0) != OK)
	{
	  *bilan = S_imu_BAD_PARAMS;
	  return ERROR;
	}
    }


  return OK;
}


/*------------------------------------------------------------------------
 * imuComputeAccelBiasesCntrl  -  control codel of EXEC request ComputeAccelBiases
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imu_BAD_PARAMS
 *                 S_imu_COMP_FAILED
 *
 * Return:    OK ou ERROR
 */

STATUS imuComputeAccelBiasesCntrl (IMU_COMPUTEACCELBIASES_PARAMS *computeAccelBiasesParams, int *bilan)
{

  if(computeAccelBiasesParams->duration <= 0.0)
    {
      printf("imuComputeAccelBiasesCntrl: duration should be > 0.0\n");
      *bilan = S_imu_BAD_PARAMS;
      return ERROR;
    }


  return OK;
}


/*------------------------------------------------------------------------
 * imuSetAccelBiasesCntrl  -  control codel of CONTROL request SetAccelBiases
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS imuSetAccelBiasesCntrl (IMU_SETACCELBIASES_PARAMS *setAccelBiasesParams, int *bilan)
{

  /* Just copy the values given by the operator */
  SDI_F->sensors.xAccelBias = setAccelBiasesParams->xAccelBias;
  SDI_F->sensors.yAccelBias = setAccelBiasesParams->yAccelBias;
  SDI_F->sensors.zAccelBias = setAccelBiasesParams->zAccelBias;

  return OK;
}


/*------------------------------------------------------------------------
 * imuComputeAngSpeedsBiasesCntrl  -  control codel of EXEC request ComputeAngSpeedsBiases
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imu_BAD_PARAMS
 *                 S_imu_COMP_FAILED
 *
 * Return:    OK ou ERROR
 */

STATUS imuComputeAngSpeedsBiasesCntrl (IMU_COMPUTEANGSPEEDSBIASES_PARAMS *computeAngSpeedsBiasesParams, int *bilan)
{

  if(computeAngSpeedsBiasesParams->duration <= 0.0)
    {
      printf("imuComputeAngSpeedsBiasesCntrl: duration should be > 0.0\n");
      *bilan = S_imu_BAD_PARAMS;
      return ERROR;
    }


  return OK;
}


/*------------------------------------------------------------------------
 * imuBackStepAttitudeEstimationCntrl  -  control codel of EXEC request BackStepAttitudeEstimation
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_imu_BAD_PARAMS
 *
 * Return:    OK ou ERROR
 */

STATUS imuBackStepAttitudeEstimationCntrl (IMU_BACKSTEPPING_PARAMS *backStepParams, int *bilan)
{

  if(backStepParams->Km <= 0.0)
    {
      printf("imuBackStepAttitudeEstimationCntrl: Km gain should be > 0.0\n");
      *bilan = S_imu_BAD_PARAMS;
      return ERROR;
    }
  if(backStepParams->Kb <= 0.0)
    {
      printf("imuBackStepAttitudeEstimationCntrl: Kb gain should be > 0.0\n");
      *bilan = S_imu_BAD_PARAMS;
      return ERROR;
    }

  if(backStepParams->Km <= backStepParams->Kb)
    printf("imuBackStepAttitudeEstimationCntrl: Warning: usually Km > Kb \n");
  /*** To check: should Km be > Kb ? ****/


  return OK;
}

/*------------------------------------------------------------------------
 * imuStopBackStepAttitudeEstimationCntrl  -  control codel of CONTROL request StopBackStepAttitudeEstimation
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS imuStopBackStepAttitudeEstimationCntrl (int *bilan)
{

  SDI_F->backStepParams.mode = IMU_BACKSTEP_OFF;

  return OK;
}


/*------------------------------------------------------------------------
 * imuStopMonitorCntrl  -  control codel of CONTROL request StopMonitor
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS imuStopMonitorCntrl (int *bilan)
{

  SDI_F->monitorData.status = IMU_MONITOR_OFF;
  return OK;
}
