/* 
 * Copyright (c) 2002 LAAS/CNRS                       --  Wed Jun 12 2002
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * $Id: imuInt.h,v 1.1 2007/04/16 14:11:06 sjoyeux Exp $
 */
#ifndef H_IMUINT
#define H_IMUINT

#define IMU_READER_PRI		200
#define IMU_READER_STACK	20000
#define IMU_READER_NAME		"imuReadTask"

#define DEVICE_NAME_LEN		16

/* G-Range: should be 2 or 10, depending on the device */
/* Accel(G) = data * GR * 1.5/2^15  */
/* On the user's manual, the value for Dala is GR (G-range) = 10 but the right value seems to be GR = 2 (zAccel = -1 when robot still and on flat terrain) */
#ifdef ATRV  /* Dala */
#define G_RANGE 2
#endif
#ifdef ATRV_JUNIOR  /* Gromit */
#define G_RANGE 10
#endif

/* Angular rate range: should be 100 or 200, depending on the device */
/* Rate(deg/s) = data * RR * 1.5/2^15  */
/* Normally, with RR (Rate Range) = 200 deg/s for the IMU400CC-200 (Dala) */
/* But the right value for Dala seems to be 100 */
#ifdef ATRV
#define ANGULAR_RATE_RANGE 100
#endif
#ifdef ATRV_JUNIOR
#define ANGULAR_RATE_RANGE 150   /* To be checked (again) ! */
/* = (200*3/4) */
#endif


/****** Macros *******************************************/
#ifndef SQ
#define SQ(a)	((a)*(a))
#endif
/*********************************************************/



/***** Structures ****************************************/

typedef struct ImuData {
   int exit;				/* reader will exit when true */
   int period;				/* reader's period */

   int fd;				/* serial line */
   int baud;				/* line speed */
   int targetBaud;			/* targeted line speed */

   /* sensor id */
   long serial;				/* serial number */
   char version[DEVICE_NAME_LEN];	/* type and firmware */

   /* instantaneous values */
   double rollRate;			/* roll rate (x-axis) rad/s */
   double pitchRate;			/* pitch rate (y-axis) rad/s */
   double yawRate;			/* yaw rate (z-axis) rad/s */

  /* Angular rates after bias compensation */
   double rollRateNoBias;			/* roll rate (x-axis) rad/s */
   double pitchRateNoBias;			/* pitch rate (y-axis) rad/s */
   double yawRateNoBias;

  /* Th:--- The same signals but filtered (if filter is ON) */
   double rollRateFiltered;		/* roll rate (x-axis) rad/s */
   double pitchRateFiltered;		/* pitch rate (y-axis) rad/s */
   double yawRateFiltered;		/* yaw rate (z-axis) rad/s */


  /* Ajout Th ------------------------------ */
  double time;   /* = period/nTicksPerSec  *****/
  /* --------------------------------------- */

   double xAccel;			/* acceleration x (m/s) */
   double yAccel;			/* acceleration y (m/s) */
   double zAccel;			/* acceleration z (m/s) */
  
  /* Accel after biases compensation */
  double xAccelNoBias;
  double yAccelNoBias;
  double zAccelNoBias;

  /* Th:--- The same signals but filtered (if filter is ON) */
   double xAccelFiltered;		/* roll rate (x-axis) rad/s */
   double yAccelFiltered;		/* pitch rate (y-axis) rad/s */
   double zAccelFiltered;		/* yaw rate (z-axis) rad/s */


   double temp;				/* temperature (C) */
   unsigned short ticks;		/* internal time */

   /* integrated values */
   double roll;
   double pitch;
   double yaw;
  /***** Thierry *******************************/
  double xSpeed;                        /* speed x (m/s) */
  double ySpeed;                        /* speed y (m/s) */
  double zSpeed;                        /* speed z (m/s) */

  double xSpeedVariance;                /* Variances for x-y-zSpeed */
  double ySpeedVariance;
  double zSpeedVariance;

  double xPos;              /* Position in x (m) */
  double yPos;              /* Position in y (m) */
  double zPos;              /* Position in z (m) */
  /* Ou plutot les Delta_x,y,z puis intgration  part? */

  double xPosVariance;      /* Variance for the Position in x */
  double yPosVariance;
  double zPosVariance;

  double rollVariance;                /* Variances for roll-pitch-yaw rate */
  double pitchVariance;
  double yawVariance;

  /***** Calulation of attitude angles at stop ***/
  double phiLama; /* rotation autour de yLama */
  double psiLama; /* Rotation autour de xLama */

  /***** Th End ********************************/

   /* command requests */
   double zero;				/* calibrate zero bias (s) */ 
} ImuData;


STATUS	imuReadTaskPerm(ImuData *imuData);



/******** Fonctions and constants from or for imuFilter.c *********/
#define Pi 3.14159

STATUS imuFilterInit (int numSamplesUsedToFilterParam,
		      float sigmaFilterParam );

#define IMU_NUM_ANGSPEEDS	3
#define IMU_OMEGA_X	0
#define IMU_OMEGA_Y	1
#define IMU_OMEGA_Z	2

STATUS imuGaussAverageFilter(double   omega_x,
			     double   omega_y,
			     double   omega_z,
			     double   angSpeedsFiltered[IMU_NUM_ANGSPEEDS] );

#define IMU_NUM_ACCEL	3
#define IMU_ACCEL_X	0
#define IMU_ACCEL_Y	1
#define IMU_ACCEL_Z	2

STATUS imuFilterInit_accel (int numSamplesUsedToFilterParam_accel,
			    float sigmaFilterParam_accel);

STATUS imuGaussAverageFilter_accel(double   accel_x,   /* Angular speeds given by IMU */
				   double   accel_y,
				   double   accel_z,
				   double   accelFiltered[IMU_NUM_ACCEL] );

/*********************************************/


/* Initialisations to be done when module starts */
STATUS imuInitGlobalVariables();

/** initImuData for Integration **/
STATUS initImuDataForIntegration();


STATUS   getVersion(ImuData *imu);


#endif /* H_IMUINT */
