/**
 ** imuMonitorTaskCodels.c
 **
 ** Codels called by execution task imuMonitorTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#ifdef VXWORKS
# include <vxWorks.h>
#else
# include <portLib.h>
#endif
#include "server/imuHeader.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include "imuStruct.h"

#include "imuInt.h"

#include "libimu.h"

/* file pointer for the monitor log */
static FILE*  file;

#define DEBUG 0



/*------------------------------------------------------------------------
 *
 * imuMonitorTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
imuMonitorTaskInit(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 *
 * imuMonitorTaskPerm  --  codel of permanent activity
 *
 * Description: 
 * 
 * Reports:      OK
 * 
 * Returns:    OK or ERROR
 */

STATUS
imuMonitorTaskPerm(int *report)
{
  /* ... add your code here ... */
  return OK;
}


/*------------------------------------------------------------------------
 * FlatTerrainMonitor
 *
 * Description: 
 *
 * Reports:      OK
 *              S_imu_BAD_PARAMS
 *              S_imu_LIB_ERROR
 */

/* imuFlatTerrainMonitorStart  -  codel START of FlatTerrainMonitor
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuFlatTerrainMonitorStart(IMU_MONITOR_PARAMS *monitorParams, int *report)
{

  /* Check */
  if(monitorParams->timewidth_nbCycles <= 0) {
    fprintf(stderr, "imuFlatTerrainMonitorStart: width of Time window not valid\n");
    *report = S_imu_BAD_PARAMS;
    return END;
  }
  
  SDI_F->monitorData.status = IMU_MONITOR_ON;
  
  /*----------- Prepare Save Logs on file if asked -------------*/
  if(monitorParams->logFlag == IMU_MONITOR_LOG_ON) {
    file = fopen(monitorParams->fileName, "w");
    if (file == NULL) {
      fprintf(stderr,"imuFlatTerrainMonitorStart: Creation of log file Failed\n");
      return END;
    }
    
    fprintf(stderr, "imuFlatTerrainMonitorStart: Logging Monitor Data in file %s \n", monitorParams->fileName);

    /* Write a first line in file containing the names of signals logged */
    fprintf(stderr, "imuFlatTerrainMonitorStart: Writing Head of Log File...");
    fprintf(file, "%s\t %s\t%s\t %s\t%s\t %s\t%s\t %s\n",
	    "time",
	    "pitchRate", "rollRate",
	    "pitchRateEnergy", "rollRateEnergy",
	    "probaPitch", "probaRoll", 
	    "behavior" );
    fprintf(stderr, " Done\n");
  }
  /*------------------------------------------------------------*/


  /* Init the computation of signal energies */
  if(imuComputeEnergyInit(monitorParams->timewidth_nbCycles) != IMU_OK) {
    fprintf(stderr, "imuFlatTerrainMonitorStart: Energy Computation Init Failed\n");
    *report = S_imu_LIB_ERROR;
    return END;
  }

  /*-- Prepare parameters for the shaping function --*/
  /* MaxFlat is the highest level of Energy correspond. to Flat Terrain */
  /* MinRough is the lowest level of Energy to get a Rough Terrain */
  SDI_F->monitorParams.sigmaSigmoid = 
    (SDI_F->monitorParams.MinRough-SDI_F->monitorParams.MaxFlat) / 3.0;
  fprintf(stderr,"imuFlatTerrainMonitorStart: SigmaSigmoid is: %lf\n", SDI_F->monitorParams.sigmaSigmoid);

  return EXEC;
}


/* imuFlatTerrainMonitorExec  -  codel EXEC of FlatTerrainMonitor
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuFlatTerrainMonitorExec(IMU_MONITOR_PARAMS *monitorParams, int *report)
{
  
    ACTIVITY_EVENT  EVENT=EXEC;

#if DEBUG
  if(SDI_F->monitorData.status == IMU_MONITOR_ON)
    fprintf(stderr, "Status = IMU_MONITOR_ON\n");
  else
    fprintf(stderr, "Status = IMU_MONITOR_OFF\n");
#endif

  if(SDI_F->monitorData.status == IMU_MONITOR_OFF)
    EVENT = END;
  
  /* Compute rollRate and pitchRate energy using Calife function */
  if(imuComputeEnergy(&(SDI_F->monitorData.pitchRateEnergy), /* output */
		      &(SDI_F->monitorData.rollRateEnergy),  /* output */
		      SDI_F->sensors.pitchRateFiltered,
		      SDI_F->sensors.rollRateFiltered,
		      monitorParams->timewidth_nbCycles) != IMU_OK) {
    fprintf(stderr, "imuFlatTerrainMonitorExec: Energy Computation Failed\n");
    *report = S_imu_LIB_ERROR;
    return END;
  }
  
  /* Shaping those values to get pseudo-probas from energy */
  SDI_F->monitorData.probaPitch = imu_sigmoid(SDI_F->monitorData.pitchRateEnergy,
					      SDI_F->monitorParams.MinRough,   /* MinRough is the lowest level of Energy to get a Rough Terrain */
					      SDI_F->monitorParams.sigmaSigmoid);
  SDI_F->monitorData.probaRoll = imu_sigmoid(SDI_F->monitorData.rollRateEnergy,
					     SDI_F->monitorParams.MinRough,
					     SDI_F->monitorParams.sigmaSigmoid);
  
  /* Get the pseudo-probas from the energy */
  SDI_F->monitorData.behavior = MAX(SDI_F->monitorData.probaPitch,
				    SDI_F->monitorData.probaRoll);


  /*----------- Save Logs on file if asked -------------*/
  if(monitorParams->logFlag == IMU_MONITOR_LOG_ON) {
    fprintf(file, "%lf\t %lf\t%lf\t %lf\t%lf\t %lf\t%lf\t %lf\n",
	    SDI_F->monitorData.time,
	    SDI_F->sensors.pitchRateFiltered, SDI_F->sensors.rollRateFiltered, /* "raw" data (ie before energy computation) */
	    SDI_F->monitorData.pitchRateEnergy, SDI_F->monitorData.rollRateEnergy,
	    SDI_F->monitorData.probaPitch, SDI_F->monitorData.probaRoll, 
	    SDI_F->monitorData.behavior );
  }

  return EVENT;
}


/* imuFlatTerrainMonitorEnd  -  codel END of FlatTerrainMonitor
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuFlatTerrainMonitorEnd(IMU_MONITOR_PARAMS *monitorParams, int *report)
{

  if(monitorParams->logFlag == IMU_MONITOR_LOG_ON) {
    fclose(file);
    fprintf(stderr, "imuFlatTerrainMonitorEnd: Log file %s closed\n", monitorParams->fileName);
    SDI_F->monitorParams.logFlag = IMU_MONITOR_LOG_OFF;    
  }
  
  SDI_F->monitorData.status = IMU_MONITOR_OFF;
  fprintf(stderr, "imuFlatTerrainMonitorEnd: Flat Monitor Stopped\n");
  
  return ETHER;
}


/* imuFlatTerrainMonitorInter  -  codel INTER of FlatTerrainMonitor
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuFlatTerrainMonitorInter(IMU_MONITOR_PARAMS *monitorParams, int *report)
{

  if(monitorParams->logFlag == IMU_MONITOR_LOG_ON) {
    fclose(file);
    fprintf(stderr, "imuFlatTerrainMonitorInter: Log file %s closed\n", monitorParams->fileName);
    SDI_F->monitorParams.logFlag = IMU_MONITOR_LOG_OFF;    
  }
  
  SDI_F->monitorData.status = IMU_MONITOR_OFF;
  fprintf(stderr, "imuFlatTerrainMonitorInter: Flat Monitor Interrupted\n");
  
  return ETHER;
}

