/*	$Id: imuRequestTaskCodels.c,v 1.3 2007/08/09 11:28:23 sjoyeux Exp $ */

/* 
 * Copyright (c) 2002 LAAS/CNRS                       --  Wed Jun 12 2002
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef UNIX
#include <vxWorks.h>
/** Th: To have clock_gettime **/
#include <timers.h>
/******/
#else
#include <portLib.h>
#endif
#include <taskLib.h>

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include "commonStructLib.h"

#include "imuStruct.h"

#include "server/imuHeader.h"
#include "imuInt.h"

#include <stdlib.h>

#include "libimu.h"
static POSTER_ID headingPosterId;

/* --- Local data ---------------------------------------------------- */

static ImuData *imuData;	/* communication with the reader */

static double previousTime;    /* time value recorded at previous cycle */


/*
 * --- imuExecTaskInit --------------------------------------------------
 *
 * IDS initialization
 */

STATUS
imuExecTaskInit(int *report)
{
  int  i;
  
   strncpy(SDI_F->device.port, IMU_DEFAULT_PORT, IMU_MAX_STRLEN);
   SDI_F->device.baud = IMU_DEFAULT_BAUD;
   SDI_F->device.period = IMU_DEFAULT_PERIOD;

   /* create and initialize shared data */
   if (commonStructCreate(sizeof(ImuData), (void **)&imuData) != OK) {
      fputs("imuInit: cannot create commonStruct\n", stderr);
      *report = S_imu_ENOMEM;
      return ERROR;
   }

   commonStructTake(imuData);
   imuData->fd = -1;		/* not opened */
   imuData->baud = B38400;	/* factory default */

   /* Initialize yaw, pitch, roll for integration */
   imuData->yaw = IMU_DEFAULT_INIT_YAW;
   imuData->pitch = IMU_DEFAULT_INIT_PITCH;
   imuData->roll = IMU_DEFAULT_INIT_ROLL;

   /***** Thierry ******/
   SDI_F->sensors.yawRateBias = IMU_DEFAULT_YAWRATE_BIAS;
   SDI_F->sensors.pitchRateBias = IMU_DEFAULT_PITCHRATE_BIAS;
   SDI_F->sensors.rollRateBias = IMU_DEFAULT_ROLLRATE_BIAS;

   imuData->xSpeed = IMU_DEFAULT_INIT_XSPEED;
   imuData->ySpeed = IMU_DEFAULT_INIT_YSPEED;
   imuData->zSpeed = IMU_DEFAULT_INIT_ZSPEED;
   
   imuData->xPos = IMU_DEFAULT_INIT_XPOS;
   imuData->yPos = IMU_DEFAULT_INIT_YPOS;
   imuData->zPos = IMU_DEFAULT_INIT_ZPOS;

   SDI_F->sensors.xAccelBias = IMU_DEFAULT_XACCEL_BIAS;
   SDI_F->sensors.yAccelBias = IMU_DEFAULT_YACCEL_BIAS;
   SDI_F->sensors.zAccelBias = IMU_DEFAULT_ZACCEL_BIAS;

   /* Default: Filter OFF */
   SDI_F->imuDataFilterParams.angularSpeedsFilterMode = IMU_FILTER_OFF;
   SDI_F->imuDataFilterParams.accelFilterMode = IMU_FILTER_OFF;
   
   /* Fill the Attitude Estimation with default values */
   /*  while Attitude Estimation is not activated */
   SDI_F->backStepParams.mode = IMU_BACKSTEP_OFF;
   SDI_F->backStepParams.Km = IMU_DEFAULT_BACKSTEPPING_KmGAIN;
   SDI_F->backStepParams.Kb = IMU_DEFAULT_BACKSTEPPING_KbGAIN;
   for(i=0; i<3; i++)
     {
       SDI_F->attitudeEstimation.gravityVector[i] = 0.0;
       SDI_F->attitudeEstimation.Bestimated[i] = 0.0;
     }
   SDI_F->attitudeEstimation.attitudeAngles[0] = 0.0;
   SDI_F->attitudeEstimation.attitudeAngles[1] = 0.0;
   /***** Th End ******/


   commonStructGive(imuData);

   /** Indicate that this is the initialisation**/
   imuInitGlobalVariables();

   return OK;
}


/*
 * --- imuExecTaskEnd ---------------------------------------------------
 *
 * Deallocate all resources
 */

STATUS
imuExecTaskEnd(void)
{
   if (imuData->fd >= 0) close(imuData->fd);
#if 0
   imuData->exit = 1;
   taskDelay(10);

   commonStructTake(imuData);
   if (imuData->fd >= 0) close(imuData->fd);
   commonStructDelete(imuData);
#endif
   return OK;
}


/*
 * --- imuExecTaskPerm --------------------------------------------------
 *
 * Update poster
 */

STATUS
imuExecTaskPerm(int *report)
{
   commonStructTake(imuData);
   if (imuData->fd >= 0) {
     /* Get yaw, pitch, roll with conversion in Degrees */
     /* --> in SDI (so in Poster Sensors) */
      SDI_F->sensors.yaw = imuData->yaw * 180/M_PI;
      SDI_F->sensors.pitch = imuData->pitch * 180/M_PI;
      SDI_F->sensors.roll = imuData->roll * 180/M_PI;

      /* Angular Speeds */
      SDI_F->sensors.yawRate = imuData->yawRate;
      SDI_F->sensors.pitchRate = imuData->pitchRate;
      SDI_F->sensors.rollRate = imuData->rollRate;

      /* Angular Speeds Filtered */
      SDI_F->sensors.yawRateFiltered = imuData->yawRateFiltered;
      SDI_F->sensors.pitchRateFiltered = imuData->pitchRateFiltered;
      SDI_F->sensors.rollRateFiltered = imuData->rollRateFiltered;

      /* Angular Speeds with bias compensation */
      SDI_F->sensors.yawRateNoBias = imuData->yawRateNoBias;
      SDI_F->sensors.pitchRateNoBias = imuData->pitchRateNoBias;
      SDI_F->sensors.rollRateNoBias = imuData->rollRateNoBias;

      /* Accel. */
      SDI_F->sensors.xAccel = imuData->xAccel;
      SDI_F->sensors.yAccel = imuData->yAccel;
      SDI_F->sensors.zAccel = imuData->zAccel;

      /* Accel. Filtered */
      SDI_F->sensors.xAccelFiltered = imuData->xAccelFiltered;
      SDI_F->sensors.yAccelFiltered = imuData->yAccelFiltered;
      SDI_F->sensors.zAccelFiltered = imuData->zAccelFiltered;

      /** Accelerations with Bias Compensation  **/
      SDI_F->sensors.xAccelNoBias = imuData->xAccelNoBias;
      SDI_F->sensors.yAccelNoBias = imuData->yAccelNoBias;
      SDI_F->sensors.zAccelNoBias = imuData->zAccelNoBias;


      /* Speeds and associated Variances */
      SDI_F->sensors.xSpeed = imuData->xSpeed;
      SDI_F->sensors.ySpeed = imuData->ySpeed;
      SDI_F->sensors.zSpeed = imuData->zSpeed;

      SDI_F->sensors.xSpeedVariance = imuData->xSpeedVariance;
      SDI_F->sensors.ySpeedVariance = imuData->ySpeedVariance;
      SDI_F->sensors.zSpeedVariance = imuData->zSpeedVariance;

      /* Position and associated Variances */
      SDI_F->sensors.xPos = imuData->xPos;
      SDI_F->sensors.yPos = imuData->yPos;
      SDI_F->sensors.zPos = imuData->zPos;
      SDI_F->sensors.xPosVariance = imuData->xPosVariance;
      SDI_F->sensors.yPosVariance = imuData->yPosVariance;
      SDI_F->sensors.zPosVariance = imuData->zPosVariance;

      /** Attitude Angles as for Lama **/
      SDI_F->sensors.phiLama = imuData->phiLama;
      SDI_F->sensors.psiLama = imuData->psiLama;

   }

   commonStructGive(imuData);

   /*
   if (!count) {
     count = 20*4;
     printf ("%5.3f %5.3f\n", SDI_F->sensors.yaw, SDI_F->sensors.roll);
   }
   else
     count--;
   */


   return OK;
}


/*
 * --- Init request -----------------------------------------------------
 *
 * Open a connexion to the DMU module.
 */

/* imuInitExec  -  codel EXEC de Init
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuInitExec(ImuDevice *device, int *report)
{
   int i;
   int readerId;
   struct termios t;

   commonStructTake(imuData);

   if (imuData->fd >= 0) {
      *report = S_imu_EINUSE;
      commonStructGive(imuData);
      return ETHER;
   }

   /* configure params */
   imuData->exit = -1; /* reader will set it to 0 after init */
   imuData->period = device->period;
   switch(device->baud) {
      case 38400:
	 imuData->targetBaud = B38400;	break;
      default:
	 *report = S_imu_EINVAL;
	 goto done;
   }

   /* open serial port */
   imuData->fd = open(device->port, O_RDWR, 0);
   if (imuData->fd < 0) {
      perror("open");
      *report = S_imu_ENOENT;
      goto done;
   }

   /* reconfigure line at last baud rate (38400 on power reset) */
   tcgetattr(imuData->fd, &t);
   cfsetospeed(&t, imuData->baud);
   cfsetispeed(&t, imuData->baud);
   t.c_cflag &= ~(CSIZE|PARENB);
   t.c_cflag |= CS8 | CLOCAL;
   t.c_iflag &= ~(ISTRIP|ICRNL|INLCR|IGNCR|IXON|IXOFF|IUCLC);
   t.c_oflag &= ~OPOST;
   t.c_lflag &= ~(ICANON|ISIG|IEXTEN|ECHO);
   t.c_cc[VMIN] = 0;
   t.c_cc[VTIME] = 1; /* blocking read, 100ms timeout */
   tcsetattr(imuData->fd, TCSANOW, &t);

   commonStructGive(imuData);

   /* spawn reader */
   readerId = 
      taskSpawn(IMU_READER_NAME, IMU_READER_PRI, 8, IMU_READER_STACK,
		imuReadTaskPerm, imuData);
   if (readerId < 0) {
      perror("taskSpawn");
      *report = S_imu_ENOENT;
      return ETHER;
   }

   /* wait for initialization */
   for(i=0; i<2; i++) {
      taskDelay(10);
      commonStructTake(imuData);
      if (imuData->exit == 0 && imuData->targetBaud == imuData->baud) {
	 *report = OK;
	 goto done;
      }
      commonStructGive(imuData);
      fputs("imuInitExec: waiting for reader\n", stderr);
   }

   commonStructTake(imuData);
   close(imuData->fd);
   imuData->fd = -1;
   fputs("imuInitExec: couldn't initialize\n", stderr);
   *report = S_imu_EINVAL;

  done:
   commonStructGive(imuData);
   return ETHER;
}


/*
 * --- Zero request -----------------------------------------------------
 *
 * Calibrate bias.
 */

/* imuZeroExec  -  codel EXEC de Zero
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
imuZeroExec(double *zero, int *report)
{
   int i;

   if (*zero <= 0.0) {
      *report = S_imu_EINVAL;
      return ETHER;
   }

   commonStructTake(imuData);

   if (imuData->fd < 0) {
      *report = S_imu_ENOTINIT;
      goto done;
   }

   imuData->zero = *zero;
   commonStructGive(imuData);

   for(i=0; i<2; i++) {
      taskDelay(10);
      commonStructTake(imuData);
      if (imuData->zero == 0.0) {
	 *report = OK;
	 goto done;
      }
      if (imuData->zero == -1.0) {
	 *report = S_imu_EINVAL;
	 goto done;
      }
      commonStructGive(imuData);
      fputs("imuInitExec: waiting for reader\n", stderr);
   }

   *report = S_imu_EINVAL;
  done:
   commonStructGive(imuData);
   return ETHER;
}


/*------------------------------------------------------------------------
 * InitIntegratedData
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_imu_BAD_PARAMS
 */

/* imuInitIntegratedDataExec  -  codel EXEC de InitIntegratedData
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuInitIntegratedDataExec (IMU_INTEGRATEDDATA_INITVALUES *initValues,
					  int *bilan)
{

  commonStructTake(imuData);
  
  imuData->roll = initValues->rollInitValue;
  imuData->pitch = initValues->pitchInitValue;
  imuData->yaw = initValues->yawInitValue;
  imuData->xSpeed = initValues->xSpeedInitValue;
  imuData->ySpeed = initValues->ySpeedInitValue;
  imuData->zSpeed = initValues->zSpeedInitValue;

  initImuDataForIntegration();

  commonStructGive(imuData);

  return ETHER;
}





/*****************************************************************/
/*---- Dump data on file ----------------------------------------*/
/*****************************************************************/


/* var globales ------------*/
static int     cyclesCounter;
static int     numSamples;  /* Number of samples of IMU data written on file */
static FILE*   file;
/*--------------------------*/


/* imuDumpImuDataOnFileStart  -  codel START de DumpImuDataOnFile
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuDumpImuDataOnFileStart (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{

  numSamples = 0;
  cyclesCounter = 0;
  file = fopen(dumpOnFileParams->fileName, "w");
  if (file == NULL)
    return FAIL;

  printf("imu: Debut Ecriture fichier IMUdump\n"); 
  /* On prépare une ligne en tête du fichier pour pouvoir écrire dessus le nombre de samples dumpés à la fin de l'operation de dump */
  fprintf(file, "Number of samples written = %d      \n", 0);
  fprintf(file,"Données de l'IMU de Dala\nPeriode = %d cycles, soit %d ms\n",  dumpOnFileParams->dumpPeriod, dumpOnFileParams->dumpPeriod*10);
  fprintf(file,"%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s \t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", "rollRate", "pitchRate", "yawRate", "rollRateFilt", "pitchRateFilt", "yawRateFilt", "roll", "pitch", "yaw", "xAccel", "yAccel", "zAccel", "xAccelFiltered", "yAccelFiltered", "zAccelFiltered", "temp", "time",  "phiLama", "psiLama",  "xAccelNoBias","yAccelNoBias","zAccelNoBias", "rollRateNoBias", "pitchRateNoBias", "yawRateNoBias", "pitch", "roll1", "roll2");
  printf("imu: Fin Ecriture en-tete\n");


  return EXEC;
}




/* imuDumpImuDataOnFileExec  -  codel EXEC de DumpImuDataOnFile
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuDumpImuDataOnFileExec (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{

  cyclesCounter++;
    if(cyclesCounter == dumpOnFileParams->dumpPeriod) /* It's time to dump */
    { 
      fprintf(file, "%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf \t%lf\t%lf\t%lf \t%lf\t%lf \t%lf\t%lf \t%lf\t%lf\t%lf\t%lf\t%lf\t%lf \t%lf\t%lf",
	      SDI_F->sensors.rollRate,
	      SDI_F->sensors.pitchRate, SDI_F->sensors.yawRate,
	      SDI_F->sensors.rollRateFiltered,
	      SDI_F->sensors.pitchRateFiltered, SDI_F->sensors.yawRateFiltered,
	      SDI_F->sensors.roll, 
	      SDI_F->sensors.pitch, SDI_F->sensors.yaw,
	      SDI_F->sensors.xAccel, 
	      SDI_F->sensors.yAccel, SDI_F->sensors.zAccel,
	      SDI_F->sensors.xAccelFiltered, 
	      SDI_F->sensors.yAccelFiltered, SDI_F->sensors.zAccelFiltered,
	      imuData->temp, imuData->time,
	      SDI_F->sensors.phiLama, SDI_F->sensors.psiLama,
	      SDI_F->sensors.xAccelNoBias, SDI_F->sensors.yAccelNoBias,
	      SDI_F->sensors.zAccelNoBias,
	      SDI_F->sensors.rollRateNoBias, SDI_F->sensors.pitchRateNoBias,
	      SDI_F->sensors.yawRateNoBias,
	      SDI_F->attitudeEstimation.attitudeAngles[0],   /* pitch */
	      SDI_F->attitudeEstimation.attitudeAngles[1]);  /* roll  */

      fprintf(file, "\n");
      numSamples++;
      /**** Remarque: Un echantillon non ecrit en entier (ligne inachevée) ne sera pas compté --> Il faudra supprimer la ligne incomplete à la main ***/

      cyclesCounter = 0;
    }

  return EXEC;
}




/* imuDumpImuDataOnFileEnd  -  codel END de DumpImuDataOnFile
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuDumpImuDataOnFileEnd (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{
  
  /* Attendre un peu pour etre sur que tout a ete ecrit dans le fichier */
/*   printf("Attente de fin d'écriture...\n"); */
/*   sleep(5); */
/*   printf("OK, on peut fermer le fichier\n"); */
  /* On note le Nombre total d'echantillons ecrits sur le fichier */
  /* sur la 1ere ligne du fichier */
  if( fseek (file, 0, SEEK_SET) != 0)
    return FAIL;
  fprintf(file, "Number of samples written = %d", numSamples);
  fclose(file);

  printf("imu: data dumped on file %s\n", dumpOnFileParams->fileName);

  return ETHER;
}



/* imuDumpImuDataOnFileInter  -  codel INTER de DumpImuDataOnFile
   Retourne:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuDumpImuDataOnFileInter (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{
  /* Attendre un peu pour etre sur que tout a ete ecrit dans le fichier */
/*   printf("Attente de fin d'écriture...\n"); */
/*   sleep(5); */
/*   printf("OK, on peut fermer le fichier\n"); */

  /* On note le Nombre total d'echantillons ecrits sur le fichier */
  /* sur la 1ere ligne du fichier */
  if( fseek (file, 0, SEEK_SET) != 0)
    return FAIL;
  fprintf(file, "Number of samples written = %d", numSamples);
  fclose(file);

  return ETHER;
}



/* imuDumpImuDataOnFileFail  -  codel FAIL de DumpImuDataOnFile
   Retourne:  FAIL ZOMBIE */
ACTIVITY_EVENT imuDumpImuDataOnFileFail (IMU_DUMPONFILE_PARAMS *dumpOnFileParams, int *bilan)
{
  

  return ETHER;
}



/**************************************************************/
/********* Thierry ********************/

/*
 * --- initImuData for Integration ----------------------------------------------------
 *
 */

STATUS
initImuDataForIntegration()
{
  imuData->xSpeed = SDI_F->initValues.xSpeedInitValue;
  imuData->ySpeed = SDI_F->initValues.ySpeedInitValue;
  imuData->zSpeed = SDI_F->initValues.zSpeedInitValue;

  imuData->xPos = SDI_F->initValues.xPosInitValue;
  imuData->yPos = SDI_F->initValues.yPosInitValue;
  imuData->zPos = SDI_F->initValues.zPosInitValue;

  imuData->roll   = SDI_F->initValues.rollInitValue;
  imuData->pitch  = SDI_F->initValues.pitchInitValue;
  imuData->yaw    = SDI_F->initValues.yawInitValue;

  return OK;
}




/* imuGetSensorsDataStart  -  codel START de GetSensorsData
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuGetSensorsDataStart (IMU_GETSENSORSDATA_PARAMS *getSensorsDataParams, int *bilan)
{
  
  return EXEC;
}


/* imuGetSensorsDataExec  -  codel EXEC de GetSensorsData
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuGetSensorsDataExec (IMU_GETSENSORSDATA_PARAMS *getSensorsDataParams, int *bilan)
{
  FILE*  file;
  char   currentFileName[IMU_DUMP_FILENAME_MAX];
  char   numFileChar[5];
  
  strcpy(currentFileName, getSensorsDataParams->baseFileName);
  sprintf(numFileChar, "%.3d", SDI_F->setSensorsDataParams.getSensorsData_numFile);
  strcat(currentFileName, numFileChar);
  file = fopen(currentFileName, "w");

  fprintf(file, " rollRate = %lf\tpitcRhate = %lf\tyawRate = %lf\n rollRateFiltered = %lf\tpitchRateFiltered = %lf\tyawRateFiltered = %lf\n roll = %lf\tpitch = %lf\tyaw = %lf\n xAccel = %lf\tyAccel = %lf\tzAccel = %lf\n xAccelFiltered = %lf\tyAccelFiltered = %lf\tzAccelFiltered = %lf\n time = %lf\n phiLama = %lf\tpsiLama = %lf \n",
	  SDI_F->sensors.rollRate,
	  SDI_F->sensors.pitchRate, SDI_F->sensors.yawRate,
	  SDI_F->sensors.rollRateFiltered,
	  SDI_F->sensors.pitchRateFiltered, SDI_F->sensors.yawRateFiltered,
	  SDI_F->sensors.roll, 
	  SDI_F->sensors.pitch, SDI_F->sensors.yaw,
	  SDI_F->sensors.xAccel, 
	  SDI_F->sensors.yAccel, SDI_F->sensors.zAccel,
	  SDI_F->sensors.xAccelFiltered, 
	  SDI_F->sensors.yAccelFiltered, SDI_F->sensors.zAccelFiltered,
	  imuData->time,
	  SDI_F->sensors.phiLama, SDI_F->sensors.psiLama );

  /* Incrementation of the number of the current file */
  SDI_F->setSensorsDataParams.getSensorsData_numFile++;
  
  fclose(file);
  
  return END;
}


/* imuGetSensorsDataEnd  -  codel END de GetSensorsData
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuGetSensorsDataEnd (IMU_GETSENSORSDATA_PARAMS *getSensorsDataParams, int *bilan)
{
  
  return ETHER;
}


/* imuGetSensorsDataFail  -  codel FAIL de GetSensorsData
   Retourne:  FAIL ZOMBIE */
ACTIVITY_EVENT imuGetSensorsDataFail (IMU_GETSENSORSDATA_PARAMS *getSensorsDataParams, int *bilan)
{
  printf("imu: imuGetSensorsData File No.%d failed\n", SDI_F->setSensorsDataParams.getSensorsData_numFile);

  return ETHER;
}




/*------------------------------------------------*/
/*- global var for ComputeAccelBiases ----------*/
static int numAccelSamplesAverage; /* Number of accel samples used to calculate the averages */
static double computeBiasesStartTime; /* The time when the request ComputeAccelBiases was started */
static double xAccelSum; /* sum of the xAccel samples used to calculate the average */
static double yAccelSum;
static double zAccelSum;
/*----------------------*/


/* imuComputeAccelBiasesStart  -  codel START de ComputeAccelBiases
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAccelBiasesStart (IMU_COMPUTEACCELBIASES_PARAMS *computeAccelBiasesParams, int *bilan)
{

  /* Get the starting time */
  computeBiasesStartTime = imuData->time;
  printf("imu: ComputeAccelBiasesStart starting time: %lf\n", computeBiasesStartTime);
  /* Init the samples counter */
  numAccelSamplesAverage = 0;

  /* Init the sums */
  xAccelSum = 0.0;
  yAccelSum = 0.0;
  zAccelSum = 0.0;

  return EXEC;
}


/* imuComputeAccelBiasesExec  -  codel EXEC de ComputeAccelBiases
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAccelBiasesExec (IMU_COMPUTEACCELBIASES_PARAMS *computeAccelBiasesParams, int *bilan)
{
  ACTIVITY_EVENT event;


  if( (imuData->time - computeBiasesStartTime) < computeAccelBiasesParams->duration )
    {
      /* Add the new sample to the sum (Must Check that there is a new sample)!!!! */
      xAccelSum += SDI_F->sensors.xAccel;  /* do it with SDI data or imuData ?? */
      yAccelSum += SDI_F->sensors.yAccel;
      zAccelSum += SDI_F->sensors.zAccel;
      
      /* Count this new sample */
      numAccelSamplesAverage++;

      event = EXEC;
    }
  else
    event = END;


  return event;
}



/* imuComputeAccelBiasesEnd  -  codel END de ComputeAccelBiases
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAccelBiasesEnd (IMU_COMPUTEACCELBIASES_PARAMS *computeAccelBiasesParams, int *bilan)
{
  ACTIVITY_EVENT event;
  double xAccelAverage, yAccelAverage, zAccelAverage;
  double xAccel_LamaFrameAverage, yAccel_LamaFrameAverage, zAccel_LamaFrameAverage;
  double xAccel_Lama_expected, yAccel_Lama_expected, zAccel_Lama_expected;
  double xAccel_expected, yAccel_expected, zAccel_expected;


  /* Calculate the averages for the accel, considering the number of samples that were recorded */
  if( numAccelSamplesAverage == 0 )
    event = FAIL;
  else
    {
      xAccelAverage = xAccelSum / ( (double) numAccelSamplesAverage);
      yAccelAverage = yAccelSum / ( (double) numAccelSamplesAverage);
      zAccelAverage = zAccelSum / ( (double) numAccelSamplesAverage);

      event = ETHER;
    }

  /* Conversion to have accel in Lama Frame */
  xAccel_LamaFrameAverage = - yAccelAverage;
  yAccel_LamaFrameAverage = xAccelAverage;
  zAccel_LamaFrameAverage = zAccelAverage;

  /* Calculation of the accels whe should have considering the Phi and Psi measured by the operator */
  xAccel_Lama_expected = sin( computeAccelBiasesParams->phiMeasured );
  yAccel_Lama_expected = - sin( computeAccelBiasesParams->psiMeasured )*cos( computeAccelBiasesParams->phiMeasured ) ;
  zAccel_Lama_expected = - cos( computeAccelBiasesParams->psiMeasured )*cos( computeAccelBiasesParams->phiMeasured );

  /* Convert this to the IMU frame */
  xAccel_expected = yAccel_Lama_expected;
  yAccel_expected = - xAccel_Lama_expected;
  zAccel_expected = zAccel_Lama_expected;


  /* The difference between the accel expected and the averages calculated are the biases to be compensated */
  SDI_F->sensors.xAccelBias = xAccelAverage - xAccel_expected;
  SDI_F->sensors.yAccelBias = yAccelAverage - yAccel_expected;
  SDI_F->sensors.zAccelBias = zAccelAverage - zAccel_expected;
  /* Check that sign is correct.... */
  
  printf("imu: xAccelBias = %lf\n", SDI_F->sensors.xAccelBias);
  printf("imu: yAccelBias = %lf\n", SDI_F->sensors.yAccelBias);
  printf("imu: zAccelBias = %lf\n", SDI_F->sensors.zAccelBias);


  return event;
}





/* imuComputeAccelBiasesFail  -  codel FAIL de ComputeAccelBiases
   Retourne:  FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAccelBiasesFail (IMU_COMPUTEACCELBIASES_PARAMS *computeAccelBiasesParams, int *bilan)
{

  printf("imu: Computation of AccelBiases Failed\n");
  printf("imu: Biases are set to the default values\n");

  SDI_F->sensors.xAccelBias = IMU_DEFAULT_XACCEL_BIAS;
  SDI_F->sensors.yAccelBias = IMU_DEFAULT_YACCEL_BIAS;
  SDI_F->sensors.zAccelBias = IMU_DEFAULT_ZACCEL_BIAS;

  
  return ETHER;
}




/*------------------------------------------------*/
/*- global var for ComputeAngSpeedsBiases ----------*/
static int numAngSpeedsSamplesAverage; /* Number of accel samples used to calculate the averages */
static double compAngSpeedsBiasesStartTime; /* The time when the request ComputeBiases was started */
static double yawRateSum; /* sum of the yawRate samples used to calculate the average */
static double pitchRateSum;
static double rollrateSum;
/*----------------------*/



/* imuComputeAngSpeedsBiasesStart  -  codel START de ComputeAngSpeedsBiases
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAngSpeedsBiasesStart (IMU_COMPUTEANGSPEEDSBIASES_PARAMS *computeAngSpeedsBiasesParams, int *bilan)
{

  /* Get the starting time */
  compAngSpeedsBiasesStartTime = imuData->time;
  printf("imu: ComputeAngSpeedsBiases starting time: %lf\n", compAngSpeedsBiasesStartTime);
  /* Init the samples counter */
  numAngSpeedsSamplesAverage = 0;

  /* Init the sums */
  yawRateSum = 0.0;
  pitchRateSum = 0.0;
  rollrateSum = 0.0;


  return EXEC;
}




/* imuComputeAngSpeedsBiasesExec  -  codel EXEC de ComputeAngSpeedsBiases
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAngSpeedsBiasesExec (IMU_COMPUTEANGSPEEDSBIASES_PARAMS *computeAngSpeedsBiasesParams, int *bilan)
{

  ACTIVITY_EVENT event;


  if( (imuData->time - compAngSpeedsBiasesStartTime) < computeAngSpeedsBiasesParams->duration )
    {
      /* Add the new sample to the sum (Must Check that there is a new sample)!!!! */
      yawRateSum   += SDI_F->sensors.rollRate;  /* do it with SDI data or imuData ?? */
      pitchRateSum += SDI_F->sensors.pitchRate;
      rollrateSum  += SDI_F->sensors.rollRate;
      
      /* Count this new sample */
      numAngSpeedsSamplesAverage++;

      event = EXEC;
    }
  else
    event = END;


  return event;

}




/* imuComputeAngSpeedsBiasesEnd  -  codel END de ComputeAngSpeedsBiases
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAngSpeedsBiasesEnd (IMU_COMPUTEANGSPEEDSBIASES_PARAMS *computeAngSpeedsBiasesParams, int *bilan)
{
  ACTIVITY_EVENT event;

  /* Calculate the averages for the accel, considering the number of samples that were recorded */
  if( numAccelSamplesAverage == 0 )
    event = FAIL;
  else
    {
      SDI_F->sensors.yawRateBias   = yawRateSum / ( (double) numAngSpeedsSamplesAverage);
      SDI_F->sensors.pitchRateBias = pitchRateSum / ( (double) numAngSpeedsSamplesAverage);
      SDI_F->sensors.rollRateBias  = rollrateSum / ( (double) numAngSpeedsSamplesAverage);

      event = ETHER;
    }

  printf("imu: yawRate Bias = %lf\n", SDI_F->sensors.yawRateBias);
  printf("imu: pitchRate Bias = %lf\n", SDI_F->sensors.pitchRateBias);
  printf("imu: rollRate Bias = %lf\n", SDI_F->sensors.rollRateBias);


  return ETHER;
}


/* imuComputeAngSpeedsBiasesFail  -  codel FAIL de ComputeAngSpeedsBiases
   Retourne:  FAIL ZOMBIE */
ACTIVITY_EVENT imuComputeAngSpeedsBiasesFail (IMU_COMPUTEANGSPEEDSBIASES_PARAMS *computeAngSpeedsBiasesParams, int *bilan)
{
  
  printf("imu: computation of AccelBiases Failed\n");
  printf("imu: biases are set to the default values\n");

  SDI_F->sensors.rollRateBias  = IMU_DEFAULT_ROLLRATE_BIAS;
  SDI_F->sensors.pitchRateBias = IMU_DEFAULT_PITCHRATE_BIAS;
  SDI_F->sensors.yawRateBias   = IMU_DEFAULT_YAWRATE_BIAS;

  return ETHER;
}


/*********** BackStepping Attitude Estimation **************/

/* imuBackStepAttitudeEstimationStart  -  codel START de BackStepAttitudeEstimation
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuBackStepAttitudeEstimationStart (IMU_BACKSTEPPING_PARAMS *backStepParams, int *bilan)
{
  int  i;

  if(backStepParams->mode == IMU_BACKSTEP_OFF)
    /* Nothing to be done */
    return END;

  /* Reset Estimated Gravity vector to [0 0 1] */
  SDI_F->attitudeEstimation.gravityVector[0] = 0.0;
  SDI_F->attitudeEstimation.gravityVector[1] = 0.0;  
  SDI_F->attitudeEstimation.gravityVector[2] = 1.0;

  /* Reset Estimated Angular Speeds biases Bestimated to [0 0 0] */
  for(i = 0; i < 3; i++)
    SDI_F->attitudeEstimation.Bestimated[i] = 0.0;


  previousTime = imuData->time;


  return EXEC;
}




/* imuBackStepAttitudeEstimationExec  -  codel EXEC de BackStepAttitudeEstimation
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuBackStepAttitudeEstimationExec (IMU_BACKSTEPPING_PARAMS *backStepParams, int *bilan)
{
  double mes_accel[3];
  double mes_gyro[3];
  double samplingTime;
  double pitch, roll;

  if(SDI_F->backStepParams.mode == IMU_BACKSTEP_OFF)
    return END;
  else
    {
      /* Conversion of the accelerations (units) from g to m/s^2 */
      mes_accel[0] = SDI_F->sensors.xAccelNoBias * IMU_GRAVITY_ACCEL;
      mes_accel[1] = SDI_F->sensors.yAccelNoBias  * IMU_GRAVITY_ACCEL;
      mes_accel[2] = SDI_F->sensors.zAccelNoBias * IMU_GRAVITY_ACCEL;
      
      /* Get the angular speeds */
      mes_gyro[0] = SDI_F->sensors.rollRateNoBias;   /* w_x */
      mes_gyro[1] = SDI_F->sensors.pitchRateNoBias;  /* w_y */
      mes_gyro[2] = SDI_F->sensors.yawRateNoBias;    /* w_z */

      /*-- NB: Should we really use the bias-compensated Data ?? --*/

      /* Calculate current sample time */
      samplingTime = imuData->time - previousTime;
      /* Add test sampling time > 0 ? */
      /* Alert if sampling time is not the expected value? (i.e. Nb ticks/s for module)*/

      /* Estimation of gravity vector */
      imuGravityEstimation(SDI_F->attitudeEstimation.gravityVector, 
			   SDI_F->attitudeEstimation.Bestimated, 
			   mes_gyro, mes_accel, 
			   backStepParams->Km, backStepParams->Kb, 
			   samplingTime);

      /* Estimated attitude angles are then derived from this gravity vector */
      imuAttitudeAngles(SDI_F->attitudeEstimation.attitudeAngles, SDI_F->attitudeEstimation.gravityVector);

      /*** TODO: Errors should be checked ***/

      pitch = SDI_F->attitudeEstimation.attitudeAngles[0];
      roll  = SDI_F->attitudeEstimation.attitudeAngles[1];

      /*----- Update the t3d exportPomSE -----*/
      POM_EULER euler = { 0, pitch, asin(sin(roll) / cos(pitch)), 0, 0, 0 };

      if (headingPosterId)
      {
	  GYRO_YAW gyro;
	  if (imuGYRO_YAWPosterRead(headingPosterId, &gyro) == ERROR)
	      return ERROR;
	  
	  euler.yaw = gyro.yaw;
      }
      SDI_F->exportPomSE.sensorToMain.euler = euler;

      /* Time value for next cycle */
      previousTime = imuData->time;

      return EXEC;
    }
}


/* imuBackStepAttitudeEstimationEnd  -  codel END de BackStepAttitudeEstimation
   Retourne:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuBackStepAttitudeEstimationEnd (IMU_BACKSTEPPING_PARAMS *backStepParams, int *bilan)
{
  int i;

  /* Reset Estimated Gravity vector to [0 0 1] */
  SDI_F->attitudeEstimation.gravityVector[0] = 0.0;
  SDI_F->attitudeEstimation.gravityVector[1] = 0.0;
  SDI_F->attitudeEstimation.gravityVector[2] = 1.0;

  /* Reset Estimated Angular Speeds biases Bestimated to [0 0 0] */
  for(i = 0; i < 3; i++)
    SDI_F->attitudeEstimation.Bestimated[i] = 0.0;

  /* Reset angles to 0.0 */
  for(i=0; i<2; i++)
    SDI_F->attitudeEstimation.attitudeAngles[i] = 0.0;
  
  SDI_F->backStepParams.mode = IMU_BACKSTEP_OFF;

  printf("imuBackStepAttitudeEstimationEnd: Attitude Estimation Stopped\n");

  return ETHER;
}

/*------------------------------------------------------------------------
 * SetHeadingPoster
 *
 * Description: 
 *
 * Reports:      OK
 *              S_imu_NOT_FOUND
 */

/* SetHeadingPosterExec  -  codel EXEC of SetHeadingPoster
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetHeadingPosterExec(GEN_STRING64* headingPosterName, int *report)
{
    if (*headingPosterName->name)
    {
	if (imuGYRO_YAWPosterRead(headingPosterName->name, &headingPosterId) == ERROR)
	    *report = S_imu_NOT_FOUND;
    }
    else
	headingPosterId = NULL;

    return ETHER;
}


/* imuGetDeviceVersionExec  -  codel EXEC de GetDeviceVersion
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT imuGetDeviceVersionExec (int *bilan)
{
  
  if (imuData->fd < 0) {
    printf("imuGetDeviceVersionExec: imu not Initialized\n");
    *bilan = S_imu_ENOTINIT;
    return ETHER;
   }

  if(getVersion(imuData) != OK)
    {
      printf("imuGetDeviceVersionExec: failed to read Device Version\n");
      *bilan = S_imu_CANNOT_READ_SERIAL;
    return ETHER;
    }

  /* If succeded, print the DMU Version obtained */
  printf("imu: DMU Version: %s\n", imuData->version);

  return ETHER;
}

